#========================================================================================
# package      DE_EPAGES::Content::API::Object::Category
# state        public
# base         DE_EPAGES::Content::API::Object::ContentObject
# description  object interface for Category.
#========================================================================================
package DE_EPAGES::Content::API::Object::Category;
use base DE_EPAGES::Content::API::Object::ContentObject;

use strict;
use DE_EPAGES::Content::API::Constants qw ( MAX_NESTING_OF_CATEGORIES );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $Category->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'Pages'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Category = DE_EPAGES::Content::API::Object::Category->insert($hInfo);
# example      my $Class = LoadClassByAlias('Category');
#               my $Category = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as category, tests feature Subcategories.
#               Throws the error 'FeatureLimitExceeded' if no subcategories are allowed.
#               Throws the error 'TooManyLevelsOfCategories' if the limit of levels
#               for category nesting is exceeded.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Category | new object | object
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $Parent = $hInfo->{'Parent'};
    my $Shop = $Parent->getSite();

    # test if we can create subcategories
    my $FeatureMaxValue = $Shop->featureMaxValue('Subcategories');

    if (not ElementOf($Shop->id, $Parent->id, $Parent->parent->id) and not $FeatureMaxValue) {
        Error( 'FeatureLimitExceeded', {
                Feature     => 'Subcategories',
                MaxValue    => 0,
                Shop        => $Shop
            });
    }

    # add category depth if parent is a category
    $hInfo->{'CategoryDepth'} = $Parent->countCategoryNesting() + 1 if ($Parent->instanceOf('Category'));

    # check for the maximum nesting of categories in case of a unity shop
    if ($Shop->get('IsUnityRegistered') and $hInfo->{'CategoryDepth'} > MAX_NESTING_OF_CATEGORIES) {
        Error( 'TooManyLevelsOfCategories', {
                MaxValue => MAX_NESTING_OF_CATEGORIES
            });
    }

    return $class->SUPER::insert($hInfo);
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Category->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function corrects CategoryDepth if parent is changed
#               after $SUPER->set is called.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID, $hAdditionalOptions) = @_;

    # set category depth if parent is changing
    my $CorrectCategoryDepth = 0;
    if (defined($hValues->{'Parent'} and $hValues->{'Parent'}->class->alias eq 'Category' ) ){
        $CorrectCategoryDepth = 1;
    }
    $self->SUPER::set($hValues, $LanguageID, $hAdditionalOptions);
    $self->recursiveCorrectCategoryDepth() if $CorrectCategoryDepth;
    return;
}

#========================================================================================
# function     createURI
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URI = $Object->createURI($Name, $Parent, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns a suggestion for a short url.
#----------------------------------------------------------------------------------------
# input        $Name | Name of category | string
# input        $Parent | parant object | object
# input        $LanguageID | id of language | integer
#----------------------------------------------------------------------------------------
# return       $URI | suggested URI | string
#========================================================================================
sub createURI {
    my $self = shift;
    my ($Name, $Parent, $LanguageID) = @_;
    return unless defined $Name;
    return if $Name eq '';

    my $URI;
    # use shortUrl from parent object if set
    my $ShortUrl = $Parent->get('URI', $LanguageID);
    if (defined $ShortUrl && !$ShortUrl eq '') {
        $URI = join '/', $ShortUrl, $Name;
    } else {
        # use path from site without startpage
        my @Parents = grep { !$_->instanceOf('StartPage') && defined $_->get('Name', $LanguageID) } @{$Parent->get('PathFromSite')};
        my $PathFromSite = join '/', map { $_->get('Name', $LanguageID) } @Parents;
        if (defined $PathFromSite && !$PathFromSite eq '') {
            $URI = join '/', $PathFromSite, $Name;
        } else {
            $URI = $Name;
        }
    }
    return $self->SUPER::createURI($URI, $Parent, $LanguageID);
}

#========================================================================================
# function     recursivSubCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSubCategories = $category->recursivSubCategories;
#----------------------------------------------------------------------------------------
# description  Returns all sub categories, recurive depth can limit bei $MaxLevel.
#----------------------------------------------------------------------------------------
# input        $MaxLevel | maximum depth of category tree | integer
#----------------------------------------------------------------------------------------
# return       $aSubCategories | list of sub categories | reference to array of object
#========================================================================================
sub recursivSubCategories {
    my $self = shift;
    my ( $MaxLevel ) = @_;

    if ( defined $MaxLevel ) {
        return [] if 0 > $MaxLevel--;
    }

    return [
        map { ( $_, @{ $_->recursivSubCategories($MaxLevel) } ) }
        @{ $self->get('SubCategories') }
    ];
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $Category->clone( $hInfo );
# syntax       $Clone = $Category->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new category object into the database as copy from old category
#               object.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with attribute values that are different in the clone object.
#                       Note that the clone cannot have the same Parent and the same Alias.
#                    | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new category object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;

    my $NewObject = $self->SUPER::clone($hInfo, $Recursive);
    my $PageType = LoadPageTypeByViewAction( ActionByName( $self->class, 'View' ) );

    foreach my $TemplateTypeFolder (@{$PageType->templateTypeFolders}){
        my $TemplateFolderName = $TemplateTypeFolder->alias;
        my $TemplateTypeName = $PageType->templateTypeAlias($self, $TemplateFolderName);
        # EPG-24215: Ugly fix to avoid native die if layout settings have default values.
        next unless defined $TemplateTypeName and $TemplateFolderName ne $TemplateTypeName;
        $PageType->setTemplateType( $NewObject, $TemplateFolderName, $TemplateTypeName );
    }
    $NewObject->updatePageCache;

    return $NewObject;
}

#========================================================================================
# function     countCategoryNesting
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CategoryLevel = DE_EPAGES::Content::API::Object::Category->countCategoryNesting;
# example      $CategoryLevel= $Category->countCategoryNesting;
#----------------------------------------------------------------------------------------
# description  Fetches the depth from the relative path of the category
#               /Shops/<ShopAlias>/Categories is the start (0)
#----------------------------------------------------------------------------------------
# return       $CategoryLevel | nesting depth of the calling category | int
#========================================================================================
sub countCategoryNesting {
    my $self = shift;

    # we fetch the depth from the relative path: /Shops/ShopName/Categories is the start (0)
    return scalar(@{$self->get('PathFromSite')})-1;
}

#========================================================================================
# function     correctCategoryDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Category->correctCategoryDepth;
#----------------------------------------------------------------------------------------
# description  corrects Attribute CategoryDepth of $Category
#               see function countCategoryNesting
#========================================================================================
sub correctCategoryDepth {
    my $self = shift;

    my $CategoryDepth=$self->countCategoryNesting();
    $self->set({'CategoryDepth'=>$CategoryDepth});
    return;
}

#========================================================================================
# function     recursiveCorrectCategoryDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Category->recursiveCorrectCategoryDepth;
#----------------------------------------------------------------------------------------
# description  corrects Attribute CategoryDepth of $Category and its subcategories
#========================================================================================
sub recursiveCorrectCategoryDepth {
    my $self = shift;

    $self->correctCategoryDepth;
    foreach my $Category (@{ $self->get('SubCategories') }) {
        $Category->recursiveCorrectCategoryDepth();
    }
    return;
}

#========================================================================================
# function     getShopUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopUrl = $Object->getShopUrl($hParams, $hOptions, $Servlet);
#----------------------------------------------------------------------------------------
# description  Construct the shop url of a given object dependend of the following parameters:
#----------------------------------------------------------------------------------------
# input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# input        $hOptions | with following keys:
#               <ul>
#                   <li>Type          - one of 'sf', 'admin', 'mobile', 'sf/mobile',
#                   if type is 'sf/mobile' a .mobile will be generated if the servlet is defined and the client
#                   is a mobile device, otherwise a .sf url is generated; defaults to 'sf' - string
#                   <li>UseSSL        - use https as protocol if SSL enabled for shop / database (optional, defaults to 0) - boolean
#                   <li>UseObjectPath - use object path rather than objectid (optional, defaults to 0) - boolean
#                   <li>AddToken      - add security token or not (optional, defaults to 0) - boolean
#                   <li>Token         - the security token (required AddToken is true) - String
#                   <li>Cookies       - add cookies to url or not (optional, defaults to 0) - boolean
#                   <li>AddCookies    - cookie values (required if AddCookies is true) - hash.ref
#                   <li>NoUTF8        - do not use utf-8 for encoding get parameters (optional, defaults to 0) - boolean
#                   <li>IsMobileServlet - is the client a mobile device or not - boolean
#               </ul>  | ref.hash
# input        $Shop | the current shop | object
# return       $url | the url | string
#========================================================================================
sub getShopUrl {
    my $self = shift;
    my ($hParams, $hOptions, $Shop) = @_;
    my $ShopUrl;

    if ($hOptions->{'Type'} ne 'admin' && $Shop->get('IsUnityRegistered')) {
        $ShopUrl = $self->get('WebUrlUnitySSL');
    } else {
        $ShopUrl = $self->SUPER::getShopUrl($hParams, $hOptions, $Shop);
    }

    return $ShopUrl;
}

1;
