use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error                 qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script                qw( RunScript );
use DE_EPAGES::Core::API::Warning               qw ( Warning );
use DE_EPAGES::Database::API::Connection        qw( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::Object::API::Factory             qw( LoadObject LoadObjectByPath);
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function     correctAllShopsCategoryDepth
# state        public
#----------------------------------------------------------------------------------------
# syntax       correctAllShopsCategoryDepth();
#----------------------------------------------------------------------------------------
# description  corrects category depth value for every category for every shop
#========================================================================================
sub correctAllShopsCategoryDepth {
    my ($ShopAlias, $Verbose) = @_;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/' . $ShopAlias);
        _correctShopsCategoryDepth($Shop);
    } else {
        # all shops
        my $dbi = GetCurrentDBHandle();
        my $aaRows = $dbi->execute('SELECT shopid FROM shop');
        my @ShopIDs = map { $_->[0] } @$aaRows;
        my $count_max  = scalar @ShopIDs;
        my $count=0;
        foreach my $ShopID( @ShopIDs ) {
            eval {
                my $Shop = LoadObject($ShopID);
                _correctShopsCategoryDepth($Shop);
            };
            if ( ExistsError()) {
                my $Error = GetError();
                print('Correct CategoryDepth for shop ' . $ShopID . " failed with error \"" . $Error->longMessage() . "\".\n\n");
            }
            $count++;
            print "$count of $count_max\n" if ($Verbose and ($count % 100 == 0 or $count == $count_max));
        }
    }
    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    return;
}


sub _correctShopsCategoryDepth {
    my ($Shop) = @_;

    my $StartPage = $Shop->existsChild('Categories')
        ? $Shop->child('Categories')
        : undef;
    if (defined $StartPage) {
        foreach my $Category (@{$StartPage->childrenByClass('Category')}) {
            $Category->recursiveCorrectCategoryDepth;
        }
    }
    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;
    my $Verbose;
    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'shop=s'        => \$ShopAlias,
        'verbose'       => \$Verbose,

    );

    usage() if $Help;

    unless( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  { correctAllShopsCategoryDepth($ShopAlias, $Verbose); },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    return;
}

sub usage {
    print <<END_USAGE;
Description:
    corrects CategoryDepth value for every category for every shop of Database <storename>

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shop       alias of a single shop                      (optional)
                (if not given, the script is preformed for all shops)
flags:
    -help       show the command line options
    -verbose    print out extra debug info

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
