use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Dictionary::API::Template;
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath LoadRootObject );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

use constant OLD_HELP_CENTER_LINK_REGEX =>
    qr#https://helpcenter.desk.com/customer/\w{0,2}/{0,1}portal/articles/2863565-tips-for-legal-pages#;

#========================================================================================
# function         PatchLegalPagesUnityHelpCenterLinks
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchLegalPagesUnityHelpCenterLinks();
#----------------------------------------------------------------------------------------
# description      Patches the Help Center Links for the legal pages of Unity shops
#========================================================================================
sub PatchLegalPagesUnityHelpCenterLinks {
    my ($ShopAlias) = @_;

    my $hNewHelpCenterLinks = _getNewHelpCenterLinks();

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            _PatchLegalPagesUnityHelpCenterLinks($Shop, $hNewHelpCenterLinks);
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                _PatchLegalPagesUnityHelpCenterLinks($Shop, $hNewHelpCenterLinks);
            };
            warn('Patch of the Help Center Links for the legal pages failed for shop ID '
                . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub _getNewHelpCenterLinks {
    my $System = LoadRootObject();

    my $hInfoSystem = $System->get(['Languages', 'HelpCenterUrlUnity']);
    my $ahLanguages = $hInfoSystem->{'Languages'};
    my $hNewHelpCenterLinks;

    foreach my $hLanguage (@$ahLanguages) {
        my $HelpLanguageID = _getHelpLanguageID($hLanguage);
        my $HelpCenterUrlUnity = $hInfoSystem->{'HelpCenterUrlUnity'};
        $HelpCenterUrlUnity =~ s/<LanguageCode2>/$HelpLanguageID/g;
        $hNewHelpCenterLinks->{$hLanguage->{'LanguageID'}}
            = $HelpCenterUrlUnity . '/legalpages';
    }

    return $hNewHelpCenterLinks;
}

sub _getHelpLanguageID {
    my ($hLanguage) = @_;

    my $LanguageCode2       = $hLanguage->{'Code2'};
    my $DictionaryXml       = $ENV{'EPAGES_CARTRIDGES'}."/DE_EPAGES/Dictionary/Templates/Dictionary.$LanguageCode2.xml";
    my $Template            = DE_EPAGES::Dictionary::API::Template->new();
    my $hDictionary         = $Template->dictionary( $DictionaryXml, $LanguageCode2 );
    my $hTranslationStrings = $hDictionary->{$LanguageCode2};

    return $hTranslationStrings->{'HelpLanguageID'};
}

sub _PatchLegalPagesUnityHelpCenterLinks {
    my ($Shop, $hNewHelpCenterLinks) = @_;

    warn ('Shop ' . $Shop->id . " does not contain any legal pages for Unity.\n" .
        'Run the script \"importLegalPagesUnity.pl\" to import them.')
        unless $Shop->existsChild('LegalPagesUnity')
            && scalar(@{$Shop->child('LegalPagesUnity')->children});

    my $aLegalPagesUnity = $Shop->child('LegalPagesUnity')->children;

    my $aShopLanguages = $Shop->get('Languages');
    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @$aShopLanguages;

    foreach my $LegalPage (@$aLegalPagesUnity) {
        foreach my $ShopLanguageID (@ShopLanguageIDs) {
            my $Text = $LegalPage->get('Text', $ShopLanguageID);
            if ($Text =~ OLD_HELP_CENTER_LINK_REGEX ) {
                $Text =~ s/${\OLD_HELP_CENTER_LINK_REGEX}/$hNewHelpCenterLinks->{$ShopLanguageID}/;
                $LegalPage->set({'Text' => $Text}, $ShopLanguageID);
            }
        }
    }
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $ShopAlias;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopalias=s' => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchLegalPagesUnityHelpCenterLinks($ShopAlias); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Patches the Help Center Links for the legal pages of Unity shops

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );