#==============================================================================
# package      DE_EPAGES::Content::Servlet::ShortURL
# description  Servlet to map domain names and short URLs to shop pages.
# state        public
#==============================================================================
package DE_EPAGES::Content::Servlet::ShortURL;
use base qw ( DE_EPAGES::Presentation::Servlet::BasePermission );

use strict;
use DE_EPAGES::Content::API::Constants qw ( $SHORTURL_REGEX );
use DE_EPAGES::Content::API::ShortUrl qw ( GetObjectByURI );
use DE_EPAGES::Object::API::Constants qw ( EXPANDWWWDOMAIN );
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw (GetLog LogDebug);
use DE_EPAGES::Core::API::Url qw ( UrlEncodeValue );
use DE_EPAGES::Core::API::IDN qw ( EncodeDomain );
use DE_EPAGES::Core::API::PerlTools qw ( trim );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Database::API::Connection qw (RunOnStore);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByGUID LoadRootObject);
use DE_EPAGES::Object::API::Domain qw ( GetUrlInfo );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer ProcessRequest);
use DE_EPAGES::WebInterface::API::Form;
use HTTP::Status qw( HTTP_NOT_FOUND );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'ShopNoDomain'   => 'Shop (#ShopAlias) does not have a domain name.',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet = DE_EPAGES::Content::Servlet::ShortURL->new;
#----------------------------------------------------------------------------------------
# description  Create a new servlet. Normally called by
#               <function DE_EPAGES::WebInterface::API::ApplicationServer::run>
#----------------------------------------------------------------------------------------
# return       $Servlet | blessed object | object
#========================================================================================
sub new {
    my $class = shift;
    return $class->SUPER::new;
}

#========================================================================================
# function     _abortAndRedirect
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $doRedirect = $self->_abortAndRedirect($DomainName, $aSecondaryDomains, $HttpHost, $URI, $UseSSL);
#----------------------------------------------------------------------------------------
# description  Used by Servlet handler. This function redirects proxy-forwarded hosts to
#               the correct shop domain, domain names without "www" to the actual
#               with-"www" domain name and/or session-less https requests to http.
#               All redirects are done using 301 headers.
#
#               If the request headers contain X-Forwarded-Host with a value not matching
#               the shop domain name the request is 301-redirected to the shop domain (EPG-20402)
#
#               If requested domain name is not the main shop domain nor the server hostname
#               then redirect to the main shop domain (EPG-15738):
#               case 1: browser:www.myshop.de   shop:myshop.de            => redirect to myshop.de
#               case 2: browser:myshop.de       shop:www.myshop.de        => redirect to www.myshop.de
#               case 3: browser:myshop.de       shop:myshop.de            => do nothing
#               case 4: browser:www.myshop.de   shop:www.myshop.de        => do nothing
#               case 5: browser:https://www.myshop.de   shop:www.myshop.de    => redirect to non-ssl
#               case 6: browser:https://myshop.de       shop:www.myshop.de    => redirect to non-ssl with www.myshop.de
#----------------------------------------------------------------------------------------
# input        $DomainName | the domain name set for the shop | string
# input        $aSecondaryDomains | list of secondary domain names for the shop | ref.array.string
# input        $HttpHost | given hostname in servlet request | string
# input        $URI | part of the URL without the hostname (path, query-string) | string
# input        $UseSSL | should ssl be used for redirect or not | boolean
# return       $doRedirect |  1 if a redirect is performed, else 0 | integer
#========================================================================================
sub _abortAndRedirect {
    my $self = shift;
    my ($DomainName, $aSecondaryDomains, $HttpHost, $URI, $UseSSL) = @_;
    my $RedirectTarget;
    my $UrlEncodedURI=UrlEncodeValue($URI);
    if ( $UseSSL)  {
        $RedirectTarget = "https://$DomainName/$UrlEncodedURI";
    } else {
        $RedirectTarget = "http://$DomainName/$UrlEncodedURI";
    }
    # workaround for apache mod_proxy reverse proxy on shops with multiple domains
    my $XForwardedHostHeader = $self->requestHeaders('X-Forwarded-Host');
    if($XForwardedHostHeader) {
        my ($LastHost) = trim((split(/,/, $XForwardedHostHeader))[-1]);
        if($DomainName ne $LastHost) {
            $self->redirect($RedirectTarget);
            return 1;
        }
    }

    my ($HttpHostNoWWW) = $HttpHost =~ /^(?:www\.)?(.*)$/x;
    if ($DomainName ne $HttpHost) {
        foreach my $SecondaryDomain (@$aSecondaryDomains) {
            if($HttpHost eq $SecondaryDomain || $HttpHostNoWWW eq $SecondaryDomain) {
                $self->redirect($RedirectTarget);
                return 1;
            }
        }
    }
    # redirect to the main shop domain? (EPG-15738)
    if(EXPANDWWWDOMAIN && $DomainName ne $HttpHost && $DomainName =~ /(www\.)?\Q$HttpHostNoWWW\E/g) {
        if(LoadRootObject()->get('DomainName') ne $HttpHost) {
            $self->redirect($RedirectTarget);
            return 1;
        }
    }

    return 0;
}

sub handleRequest {
    my $self = shift;

    # $HttpHost is the name received by the web server, i.e. the IDNA encoded form
    my $HttpHost = $self->{'Params'}->{'DomainName'}; # as defined in Servlet.conf
    if ($HttpHost =~ /^(.+?):(80|443)$/x) {
        $HttpHost = $1; # remove port
    }

    my $SiteGUID = $self->requestHeaders('X-ePages-Site');
    die 'Missing request header X-ePages-Site' unless defined $SiteGUID;

    my $Form  = eval { DE_EPAGES::WebInterface::API::Form->new( $self->request ) };
    if(ExistsError()){
        my $Error = GetError();
        if($Error->code eq 'FALSE_URL_ENCODING'){
            $self->handleFalseUrlEncoding($self->request->uri->as_string);
            return;
        } else {
            Error($Error);
        }
    }

    return unless $self->checkDatabaseRedirect();

    my $URI = $Form->queryValue('URI');

    #--- get webserver script name from Store DB
    my ($NewURI, $ScriptName, $DomainName);

    my $StoreName =  $self->storeName;
    # Abort request after RunOnStore.
    my $AbortRequest = 0;

    RunOnStore( 'Store' => $StoreName, 'Sub' => sub {

        my $Shop = LoadObjectByGUID( $SiteGUID );

        $DomainName = EncodeDomain($Shop->get('DomainName')); # IDN Encoded
        my @SecondaryDomains = map { EncodeDomain($_) } @{$Shop->get('SecondaryDomains')};

        # info from site and shop
        my ($hSite) = GetUrlInfo($Shop);
        # is a certificate for the shop available?
        my $ShopHasSSLCertificate = $hSite->{'HasSSLCertificate'};

        my $UseSSL;
        if ( $ShopHasSSLCertificate && $self->useSSLForShopDomains) {
            $UseSSL = 1;
        }
        if($self->_abortAndRedirect($DomainName, \@SecondaryDomains, $HttpHost, $URI, $UseSSL)) {
            $AbortRequest = 1;
            return;
        }

        $ScriptName = $Shop->get('WebServerScriptName');
        $self->response->header('X-Store', $StoreName);
        GetLog->debug('connected to store ', $StoreName);

        Error('ShopNoDomain', { 'ShopAlias' => $Shop->get('Alias') }) unless defined $DomainName;

        my %Params = (
            Object => $Shop,
            URI => $URI,
            NewURI => undef,
        );
        TriggerHook('ShortUrlRedirect', \%Params);

        my $RedirectURL;

        if (defined $Params{'NewURI'}) {
            $NewURI = $Params{'NewURI'};
        }
        else {
            my ($ContentObject, $LocaleID);

            # if no URI then start page
            if ($URI eq "") {
                $NewURI = $ScriptName . '/?ObjectPath=' . UrlEncodeValue($Shop->get('Path'));
            }
            # else if not valid shortURL then redirect to start page
            elsif ($URI !~ $SHORTURL_REGEX) {
                $RedirectURL = $UseSSL ? "https://$DomainName" : "http://$DomainName";
            }
            # else get my ContentObject
            else {
                ($ContentObject, $LocaleID) = GetObjectByURI($Shop, $URI);

                # if shortURL exists then show this
                if (defined $ContentObject) {
                    $NewURI = $ScriptName . '/?ObjectPath=' . UrlEncodeValue($ContentObject->get('Path'));
                    $NewURI = $ScriptName . '/' . $LocaleID . '/?ObjectPath=' . UrlEncodeValue($ContentObject->get('Path')) if (defined $LocaleID);
                }
                # if shortURL does not exists or its a Google Analytics request make the same like in past
                else {
                    $RedirectURL = $UseSSL ? "https://$DomainName" : "http://$DomainName";

                    # Hook and implementation of: EPG-28584.
                    my $hParams = {
                        'Object' => $Shop,
                        'URI'    => $URI,
                        'URL'    => undef,
                    };

                    TriggerHook('BeforeRedirectToStartPage', $hParams);
                    $RedirectURL = $hParams->{'URL'} if (defined $hParams->{'URL'});
                }
            }

            if (defined $RedirectURL) {
                $self->response->headers->header('Cache-Control' => 'no-cache');
                $self->redirect($RedirectURL);
                $AbortRequest = 1;
            }

            if (defined $LocaleID) {
                $NewURI .= '&Locale=' . $LocaleID;
            }
        }
    });

    return if $AbortRequest;

    my $EncodedHttpHost = EncodeDomain($DomainName); # IDNA umlaut domains

    $self->request->uri($NewURI);
    $self->request->header('Host', $EncodedHttpHost);
    my $hServerVariables = GetAppServer->serverVariables;
    $hServerVariables->{'SCRIPT_NAME'} = $ScriptName;
    $hServerVariables->{'HTTP_HOST'} = $EncodedHttpHost;

    ProcessRequest($self->request, $self->response);

    return;
}

#========================================================================================
# function     handleFalseUrlEncoding
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->handleFalseUrlEncoding( $Url );
#----------------------------------------------------------------------------------------
# description  Handles the error "FalseUrlEncoding".
#               This function sets the HTTP status 404 - "Not Found", displays
#               an error message and logs a warning message containing QUERY_STRING
#               and HTTP_REFERER.
#----------------------------------------------------------------------------------------
# input        $URL | url | string
#========================================================================================
sub handleFalseUrlEncoding {
    my $self = shift;
    my ($Url) = @_;
    TestString( 'Url', $Url );

    $self->logUserFault('URL::FalseEncoding', sprintf('URI %s has a false encoding.', $Url, "\n"));
    $self->viewError(sprintf('Url %s not found', $Url));
    return;
}

#========================================================================================
# function     logUserFault
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->logUserFault( $Code, $Message );
#----------------------------------------------------------------------------------------
# description  Logs a message using the log level "WARN" and log category 'USERFAULT::'.$Code
#----------------------------------------------------------------------------------------
# input        $Code | category code | string
# input        $Message | log message | string
#========================================================================================
sub logUserFault {
    my $self = shift;
    my ($Code, $Message) = @_;
    TestString( 'CategoryCode', $Code );
    TestString( 'UserFaultMessage', $Message );

    $Message .= "\n";
    $Message .= 'URI=' . $self->request->uri . "\n";
    my $hServerVariables = $self->serverVariables;
    foreach my $var (qw(HTTP_USER_AGENT HTTP_REFERER HTTP_HOST)) {
        my $Value = $hServerVariables->{$var};
        $Message .= "$var=$Value\n" if defined $Value && $Value ne '';
    }
    GetLog('USERFAULT::'.$Code)->warn( $Message );
}

#========================================================================================
# function     viewError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->viewError( $Message );
#----------------------------------------------------------------------------------------
# description  This function sets the HTTP status 404 - "Domain Not Found"
#               and shows an error message.
#----------------------------------------------------------------------------------------
# input        $Message | error message | string
#========================================================================================
sub viewError {
    my $self = shift;
    my ($Message) = @_;
    TestString( 'ErrorMessage', $Message );

    $self->response->code( HTTP_NOT_FOUND );
    $self->response->header('Content-Type', 'text/html');
    $self->response->content(sprintf('<html><head><title>Not Found</title></head><body>%s</a></body></html>', $Message));
    return;
}

sub init {
    my $self = shift;
    my ($hParams) = @_;
    LogDebug('hParams', $hParams);

    $self->{'Params'}->{$_} = $hParams->{lc $_} foreach qw (DomainName);

    LogDebug('self->Params', $self->{'Params'});
    return;
}

1;
