#========================================================================================
# package      DE_EPAGES::Database::API::DBImysql
# base         DE_EPAGES::Database::API::DBI
# description  MySQL specific implementation of DE_EPAGES::Database::API::DBI.
#========================================================================================
package DE_EPAGES::Database::API::DBImysql;
use base qw (DE_EPAGES::Database::API::DBI);

use strict;
use DateTime::Format::MySQL;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Iterator;
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestString );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Constants qw ( NO_SUCH_TABLE );
use Readonly;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'USERNAME_TOO_LONG' => 'Username must not exceed 15 chars!',
        'NO_GRANT_PRIV' => 'Backup database user must own the grant privilege!',
        'NO_INNODB' => 'Database engine InnoDB must be enabled!',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Database::API::DBImysql->new( $DataSource, $User, $Password, $CacheKey );
# example      DE_EPAGES::Database::API::DBImysql->new( 'dbi:mysql:storedb', 'root', 'geheim', 'DAL' );
#----------------------------------------------------------------------------------------
# description  Creates a database connection using the supplied connection parameters.
#----------------------------------------------------------------------------------------
# input        $DataSource | DBI connection string without user name/password | string
# input        $User | Database user or login name | string
# input        $Password | Database user password | string
# input        $CacheKey | key to identify the connection | string
# return       $dbi | database handle | DE_EPAGES::Database::API::DBImysql
#========================================================================================
sub new {
    my $class = shift;
    my ($DataSource, $User, $Password, $CacheKey) = @_;

    my $self = $class->SUPER::new($DataSource, $User, $Password, $CacheKey);
    $self->{'datetime_parser'} = DateTime::Format::MySQL->new;

    # communicate with the server in UTF-8, otherwise the server will convert strings
    # that are already utf-8 again
    $self->dbh->do('SET NAMES \'utf8\'');
    # set default engine to InnoDB; this way we do not need settings in my.cnf or in
    # each .sql file
    my $MySQLVarName = $self->execute('show variables like "default_storage_engine"')->[0]->[0];
    my $SQL = sprintf('SET %s=InnoDB', $MySQLVarName ? $MySQLVarName : 'storage_engine');
    $self->dbh->do($SQL);

    # get max text length
    $self->{'maxTextLength'} = $self->execute('SELECT @@max_allowed_packet')->[0]->[0]; ##no critic(RequireInterpolationOfMetachars)

    return $self;
}

#========================================================================================
# function     databaseName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $databaseName = $dbi->databaseName
#----------------------------------------------------------------------------------------
# description  Returns the name of the currently used database.
#----------------------------------------------------------------------------------------
# return       $databaseName | database name | string
#========================================================================================
sub databaseName {
    my $self = shift;

    my $aaResult = $self->execute('SELECT DATABASE()');
    return $aaResult->[0][0];
}

#========================================================================================
# function     iterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $dbi->iterator( $SQL );
# syntax       $Iterator = $dbi->iterator( $SQL, $aParams );
# syntax       $Iterator = $dbi->iterator( $SQL, $aParams, $aDttps, $aColumnNames );
# example      $Iterator = $dbi->iterator( 'SELECT languageid, code2 FROM language WHERE isused = ?',
#                                           [ 1 ], ['Integer', 'String'], ['LanguageID', 'Code2'] );
#               while( $hLanguage = <$Iterator> ) {
#                   print "ID: $hLanguage->{LanguageID}, Code: $hLanguage->{Code2}\n";
#               }
#----------------------------------------------------------------------------------------
# description  Executes an SQL statement and returns an iterator to fetch the results.
#----------------------------------------------------------------------------------------
# input        $SQL | SQL statement | string
# input        $aParams | (optional) list of parameters to replace '?' placeholders in
#                           the SQL string | ref.array.string
# input        $aDttps | (optional) list data type position of return values
#                           (see <function unquoteResultSet> | ref.array.string
# input        $aColumnNames | (optional) list of field names in the result set
#                           (see <function unquoteResultSet> | ref.array.string
# return       $Iterator | resultset iterator | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub iterator {
    my $self = shift;
    my ($SQL, $aParams, $aDttps, $aColumnNames) = @_;

    local $self->{'cache_sth'} = 0; # Can't use cached statements for iterators
    my $sth = $self->executeStatement($SQL, $aParams, $aDttps, $aColumnNames);
    my $Count = $self->execute('SELECT FOUND_ROWS()')->[0]->[0];

    return DE_EPAGES::Core::API::Iterator->new(
        Pull => sub { $sth->fetchrow_arrayref },
        Count => $Count,
        Handle => sub {
            my $Row = $self->unquoteResultSet( [ $_[1] ], $aDttps, $aColumnNames )->[0];
            LogDebug('Result', $Row, __PACKAGE__ . '::execute::Result');
            return $Row;
        },
        Destroy => sub {
            $sth->finish;
        }
    );
}

#========================================================================================
# function     lockTables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->lockTables( $ahTables, $cCode );
# example      $dbi->lockTables( [
#                   { 'table' => 'test', 'mode' => 'WRITE'},
#                   { 'table' => 'test2', 'mode' => 'WRITE'},
#               ], sub {
#                   $dbi->do('DELETE FROM test2 WHERE id = ?', $id);
#                   $dbi->do('DELETE FROM test WHERE id = ?', $id);
#               });
#----------------------------------------------------------------------------------------
# description  Locks the specified tables during execution of a code block.
#----------------------------------------------------------------------------------------
# input        $ahTables | tables and locking modes. Hash keys are:
#                   <ul>
#                   <li>table - table name - string</li>
#                   <li>mode - locking mode (READ|WRITE) - string</li>
#                   </ul> | ref.array.hash
# input        $cCode | code block | code ref
#========================================================================================
sub lockTables {
    my $self = shift;
    my ( $ahTables, $cCode ) = @_;

    $self->do( 'LOCK TABLES ' . join( ', ', map {"$_->{'table'} $_->{'mode'}"} @$ahTables ) );
    eval { $cCode->() };
    if ( ExistsError() ) {
        my $Error = GetError();
        eval { $self->do('UNLOCK TABLES') }; ##no critic qw(RequireExistsError)
        Error($Error);
    }
    else {
        $self->do('UNLOCK TABLES');
    }

    return;
}

#========================================================================================
# function     existsTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $DBI->existsTable( $Table );
# example      if( $DBI->existsTable( 'sysobjects' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the database table exists.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# return       $Exists | true if the table exists | boolean
#========================================================================================
sub existsTable {
    my $self = shift;
    my ($Table) = @_;
    my $dbh = $self->dbh;
    my $sth = $dbh->table_info(undef, undef, $Table);
    my $aResult = $sth->fetchall_arrayref;
    $sth->finish;

    # Workaround for DBD::mysql bug, where table_info always returns all tables
    return scalar( grep { $_->[2] eq $Table } @$aResult ) ? 1 : 0;
}

#========================================================================================
# function     renameTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->renameTable( $OldName, $NewName );
# example      $dbi->renameTable( 'land', 'country' );
#----------------------------------------------------------------------------------------
# description  Renames a table.
#----------------------------------------------------------------------------------------
# input        $OldName | old table name | string
# input        $OldName | new table name | string
#========================================================================================
sub renameTable {
    my $self = shift;
    my ($OldName, $NewName) = @_;

    $self->do("ALTER TABLE $OldName RENAME AS $NewName");
    return;
}

#========================================================================================
# function     renameTableColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->renameTableColumn( $Table, $OldColumnName, $NewColumnName );
# example      $dbi->renameTableColumn( 'person', 'prename', 'firstname' );
#----------------------------------------------------------------------------------------
# description  Renames a table column.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $OldColumnName | old table column name | string
# input        $NewColumnName | new table column name | string
#========================================================================================
sub renameTableColumn {
    my $self = shift;
    my ($Table, $OldColumnName, $NewColumnName) = @_;

    my $Definition = $self->columnDefinition($Table,$OldColumnName);

    $self->do("ALTER TABLE $Table CHANGE $OldColumnName $NewColumnName $Definition");
    return;
}

#========================================================================================
# function     columnDefinition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ColumnDef = $dbi->columnDefinition( $Table, $ColumnName );
# example      $ColumnDef = $dbi->columnDefinition( 'person', 'firstname' );
#----------------------------------------------------------------------------------------
# description  Returns the column definition of a table column.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $OldName | new table column name | string
# return       $ColumnDef | column definition string, e.g. 'firstname VARCHAR(30) NUT NULL' | string
#========================================================================================
sub columnDefinition {
    my $self = shift;
    my ($Table, $ColumnName) = @_;

    my $DBResult = $self->execute(<<END_SQL, [$Table, $ColumnName]);
SELECT COLUMN_TYPE, IS_NULLABLE, COLUMN_DEFAULT, EXTRA FROM INFORMATION_SCHEMA.COLUMNS
WHERE TABLE_SCHEMA=DATABASE()
AND TABLE_NAME = ?
AND COLUMN_NAME = ?
END_SQL

    my $ColumnType = $DBResult->[0][0];
    my $Null = $DBResult->[0][1];
    my $Default = $DBResult->[0][2];
    my $Extra = $DBResult->[0][3];

    $Default = $self->quoteString($Default) if ($ColumnType =~ /char/ or $ColumnType =~ /date/);

    my $Result = "$ColumnType";
    $Result .= ' not' if ($Null eq 'NO');
    $Result .= ' null';
    $Result .= " default $Default" if (defined $Default);
    $Result .= ' auto_increment' if ($Extra =~ /auto_increment/);

    return $Result;
}


#========================================================================================
# function     existsTableColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsTableColumn( $Table, $Column );
# example      if( $dbi->existsTableColumn( 'sysobjects', 'name' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the database table column exists. It should be noted that
#               this implementation currently works only if ANSI_QUOTES is not activated.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $Column | column name | string
# return       $Exists | true if the column exists | boolean
#========================================================================================
sub existsTableColumn {
    my $self = shift;
    my ($Table, $Column) = @_;

    my $CreateTable;
    eval {
        $CreateTable = $self->execute( "SHOW CREATE TABLE $Table" )->[0]->[1];
    };
    if (ExistsError()) {
        my $Error = GetError();
        return 0 if ($Error->code eq 'DBIExecute' && $Error->vars->{'code'} == NO_SUCH_TABLE);
        Error($Error);
    }

    return $CreateTable =~ m/\s`$Column`\s/ ? 1 : 0;
}

#========================================================================================
# function     existsIndex
#----------------------------------------------------------------------------------------
# syntax       $exists = $dbi->existsIndex($Table, $IndexName);
#----------------------------------------------------------------------------------------
# example      if( $dbi->existsIndex('product', 'i_product_super') ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the index exists. It should be noted that only explicitly
#               created indices can be found with the current implementation. Please
#               have a look at the MySQL developer documentation
#               (http://dev.mysql.com/doc/) in order to retrieve information on detecting
#               implicit indices, which exist for primary and foreign keys. Furthermore,
#               this implementation currently works only if ANSI_QUOTES is not activated.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $IndexName | index name | string
# return       $exists | true if the index exists | boolean
#========================================================================================
sub existsIndex {
    my $self = shift;
    my ($Table, $IndexName) = @_;

    my $CreateTable = $self->execute( "SHOW CREATE TABLE $Table" )->[0]->[1];
    return $CreateTable =~ /KEY\s+`$IndexName`/ ? 1 : 0;
}

#========================================================================================
# function     processID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProcessID = $dbi->processID;
#----------------------------------------------------------------------------------------
# description  Returns the ID of the current database process.
#----------------------------------------------------------------------------------------
# return       $ProcessID | process id | integer
#========================================================================================
sub processID {
    my $self = shift;
    return $self->execute( 'SELECT CONNECTION_ID()', [], ['Integer'] )->[0]->[0];
}

#========================================================================================
# function     currentDateTime
#----------------------------------------------------------------------------------------
# syntax       $DatabaseTime = $dbi->currentDateTime();
#----------------------------------------------------------------------------------------
# description  Returns the current time of the database server.
#----------------------------------------------------------------------------------------
# return       $DatabaseTime | current datetime of database server | DateTime
#========================================================================================
sub currentDateTime {
    my $self = shift;
    return $self->execute( 'SELECT NOW()', [], ['DateTime'] )->[0]->[0];
}

#========================================================================================
# function     existsDatabase
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsDatabase($DBName);
#----------------------------------------------------------------------------------------
# description  Returns true if the database exists.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
# return       $Exists | database exists | boolean
#========================================================================================
sub existsDatabase {
    my $self = shift;
    my ($DBName) = @_;

    my $aRows = $self->execute('SHOW DATABASES');
    my @Databases = map { $_->[0] } @$aRows;
    return scalar grep { $_ eq $DBName } @Databases;
}

#========================================================================================
# function     existsForeignKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsForeignKey( $Name );
# example      if( $dbi->existsForeignKey( 'sp_InsertAttribute' ) ) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the foreign key exists.
#----------------------------------------------------------------------------------------
# input        $ForeignKey | foreign key name | string
# return       $Exists | true if the foreign key exists | boolean
#========================================================================================
sub existsForeignKey {
    my $self = shift;
    my ($Name) = @_;

    my $Exists;
    eval {
        $Exists = $self->_fast_existsForeignKey( $Name );
    };
    if( ExistsError() ) {
        my ($Error) = GetError();
        if( $Error->code eq 'NoTableNameFound'  # table doesn't exist
         || $Error->code eq 'DBIExecute' && $Error->vars->{'code'} == NO_SUCH_TABLE) {
            $Exists = $self->_slow_existsForeignKey( $Name );
        }
        else {
            Error( $Error );
        }
    };

    return $Exists;
}

# relies on naming scheme fk_tablename_xxx, but is much faster than INFORMATION_SCHEMA
sub _fast_existsForeignKey {
    my $self = shift;
    my ($ForeignName) = @_;

    if( $ForeignName =~ m/^fk_(\w+?)_/ ) {
        local $SIG{'__WARN__'} = sub {}; # EPG-23375: catch warning: Table '...' doesn't exist
        return $self->existsForeignKeyByName($1, $ForeignName);
    }
    else {
        Error('NoTableNameFound');
    }
}

sub _slow_existsForeignKey {
    my $self = shift;
    my ($Name) = @_;

    my $ResultCount = $self->execute( <<'END_SQL', [$Name] )->[0]->[0];
SELECT COUNT(CONSTRAINT_NAME)
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS
WHERE CONSTRAINT_SCHEMA = DATABASE()
  AND CONSTRAINT_NAME = ?
END_SQL

    return $ResultCount;
}

#========================================================================================
# function     existsForeignKeyByName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsForeignKeyByName( $Table, $ForeignKey );
# example      if ($dbi->existsForeignKeyByName('product', 'fk_product_manufacturerid') {
#                 ...
#               };
#----------------------------------------------------------------------------------------
# description  Returns true if the given foreign key exists on the table. It should be
#               noted that this implementation currently works only if ANSI_QUOTES is not
#               activated.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $ForeignKey | name of a foreign key 'fk_<table>_<column>' | string
# return       $Exists | true if the given foreign key exists | boolean
#========================================================================================
sub existsForeignKeyByName {
    my $self = shift;
    my ($Table, $ForeignKey) = @_;

    my $CreateTable = $self->execute( "SHOW CREATE TABLE $Table" )->[0]->[1];
    return ($CreateTable =~ /CONSTRAINT\s`$ForeignKey`\sFOREIGN KEY/) ? 1 : 0;
}

#========================================================================================
# function     existsAnyForeignKeyOn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $dbi->existsAnyForeignKeyOn( $Table, $aColumns, $RefTable, $aRefColumns );
# example      if ($dbi->existsAnyForeignKeyOn(
#                 'product',
#                 [
#                   'manufacturerid'
#                 ],
#                 'productmanufacturer',
#                 [
#                   'manufacturerid'
#                 ]) { ... };
#----------------------------------------------------------------------------------------
# description  Returns true if the given foreign key exists. It should be noted that
#               this implementation currently works only if ANSI_QUOTES is not activated.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $aKeys | list of columns in the table | ref.array.string
# input        $RefTable | referenced table name | string
# input        $aRefKeys | list of columns in the referenced table | ref.array.string
# return       $Exists | true if the given foreign key exists | boolean
#========================================================================================
sub existsAnyForeignKeyOn {
    my $self = shift;
    my ($Table, $aKeys, $RefTable, $aRefKeys) = @_;

    my $Constraint = qr/CONSTRAINT.+FOREIGN KEY/;
    my $References = qr/REFERENCES/;

    my $CreateTable = $self->execute( "SHOW CREATE TABLE $Table" )->[0]->[1];
    my $ForeignKeys = join(',', map { "`$_`" } @{$aKeys});
    my $ReferencedKeys = join(',', map { "`$_`" }@{$aRefKeys});

    # Example: CONSTRAINT `fk_product_manufacturerid` FOREIGN KEY (`manufacturerid`) REFERENCES `productmanufacturer` (`manufacturerid`)
    if ($CreateTable =~ qr{
            $Constraint\s\($ForeignKeys\)\s
            $References\s`$RefTable`\s\($ReferencedKeys\)
        }x) {
        return 1;
    }

    return 0;
}

#========================================================================================
# function     createDatabase
#----------------------------------------------------------------------------------------
# syntax       $dbi->createDatabase($DBName);
# example      $dbi->createDatabase('storedb');
#----------------------------------------------------------------------------------------
# description  Creates a new database; sets the charset to utf-8
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
#========================================================================================
sub createDatabase {
    my $self = shift;
    my ($DBName) = @_;

    # Set up standard collation and character set; we have to use utf-8 here to ensure
    # proper display of non-latin based character sets like russian.
    # Using utf8, we do not have the chance to set up an all-purpose collation; therefore we use
    # utf8_bin here, which sorts strings by their bytes, enabling case sensitive search
    # and differneciation between e.g. german umlauts and their basic letter. That is,
    # a search for "br" will bring up results whith exactly this phrase in their names
    # or whatever, but NO results with "bar" or "Br".
    # EPG-14358
    $self->do( sprintf 'CREATE DATABASE %s CHARACTER SET utf8 COLLATE utf8_bin', $DBName );
    return;
}

#========================================================================================
# function     createEpagesDatabase
#----------------------------------------------------------------------------------------
# syntax       $dbi->createEpagesDatabase($DBName, $DBLogin, $hOptions);
# example      $dbi->createEpagesDatabase('sitedb', 'usr_sitedb');
# example      $dbi->createEpagesDatabase('storedb');
#----------------------------------------------------------------------------------------
# description  Creates a database with epages default sizes and names.
#----------------------------------------------------------------------------------------
# input        $DBName | database name | string
# input        $DBLogin | database login (default: current user) | string
# input        $hOptions | <ul>
#                           <li>DBPassword - password for user $DBLogin (default: 'epages') - string</li>
#                           </ul> | ref.hash
#========================================================================================
sub createEpagesDatabase {
    my $self = shift;
    my ( $DBName, $DBLogin, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;

    my @SupportedEngines = map {$_->[1] =~ /YES|DEFAULT/ && $_->[0]} @{$self->execute('SHOW ENGINES')};
    Error('NO_INNODB') unless 'InnoDB' ~~ @SupportedEngines;

    Error('NO_GRANT_PRIV') unless $self->hasGrantPrivilege();

    $DBLogin = 'usr_storedb' unless defined $DBLogin;
    $self->createDatabase($DBName);

    if ( defined $DBLogin ) {
        # create a user if the login does not exist
        my $aRows = $self->execute( 'SELECT 1 FROM mysql.user WHERE user = ?', [$DBLogin] );
        if ( 0 == scalar @$aRows ) {
            my $Password = $hOptions->{'DBPassword'};
            $Password = 'epages' unless $Password;
            # see EPG-17831
            $self->do( "CREATE USER '$DBLogin'\@'localhost' IDENTIFIED BY '$Password'" );
            $self->do( "CREATE USER '$DBLogin' IDENTIFIED BY '$Password'" );
        }

        # grant permissions to user
        $self->do( "GRANT ALL ON $DBName.* TO '$DBLogin'\@'localhost'" );
        $self->do( "GRANT ALL ON $DBName.* TO '$DBLogin'" );
    }

    return;
}

#========================================================================================
# function     hasGrantPrivilege
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hasGrant = $dbi->hasGrantPrivilege()
#----------------------------------------------------------------------------------------
# description  Returns true, if the current dbi user owns the grant privilege for
#               giving access rights to new users.
#----------------------------------------------------------------------------------------
# return       $hasGrant| true, if grant privilege is owned, false otherwise| Boolean
#========================================================================================
sub hasGrantPrivilege {
    my $self = shift;

    my $aRows = $self->execute( 'SHOW GRANTS' );
    foreach my $Row (@$aRows) {
        ## no critic (RequireExtendedFormatting)
        return 1 if( $Row->[0] =~ m/GRANT ALL PRIVILEGES ON \*\.\* TO '.*' WITH GRANT OPTION/);
        ##use critic
    }
    return 0;
}

#========================================================================================
# function     createUserAndStoreDBName
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($DBName, $UserName, $Password ) = $dbi->createUserAndStoreDBName( $StoreName )
#----------------------------------------------------------------------------------------
# description  Generates a new username, password and a database name for the store.
#               The epages standard format for the db name is used, the users name
#               if like the database name prefixed with 'usr_', the passwort is 'epages'.
#----------------------------------------------------------------------------------------
# input        $StoreName | Name of the store | String
# return       $DBName |  Database name | String
# return       $UserName |  Database name | String
# return       $Password |  Password | String
#========================================================================================
sub createUserAndStoreDBName {
    my $self = shift;
    my ($StoreName) = @_;

    my ($DBName, $UserName, $Password ) = $self->SUPER::createUserAndStoreDBName( $StoreName );
    Readonly::Scalar my $MaxUserNameLength => 14;
    if (length($UserName) >= $MaxUserNameLength) {
        $UserName = substr($UserName, 0, $MaxUserNameLength);
        Warning('USERNAME_TOO_LONG');
    }
    return ($DBName, $UserName, $Password );
}

#========================================================================================
# function     modifyTableColumnReplaceDefault
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnReplaceDefault($table, $column ,$defaultvalue);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $column = 'columnname';
#               $defaultvalue = 0 use functions quote* to quote the value before
#----------------------------------------------------------------------------------------
# description  Modifies a default of an existing column $column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
# input        $defaultvalue | default value of the column | string
#========================================================================================
sub modifyTableColumnReplaceDefault {
    my $self = shift;
    my ( $table, $column, $defaultvalue ) = @_;
    $self->do( "ALTER TABLE $table ALTER $column SET DEFAULT $defaultvalue", [] );
    return;
}

#========================================================================================
# function     modifyTableColumnDropDefault
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnDropDefault($table, $column );
#----------------------------------------------------------------------------------------
# description  Deletes a Default of an existing column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub modifyTableColumnDropDefault {
    my $self = shift;
    my ( $table, $column ) = @_;

    $self->do( "ALTER TABLE $table ALTER $column DROP DEFAULT", [] );
    return;
}

#========================================================================================
# function     modifyTableColumnDropAutoIncrement
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnDropAutoIncrement($table, $column );
#----------------------------------------------------------------------------------------
# description  Deletes auto_increment of an existing column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub modifyTableColumnDropAutoIncrement {
    my $self = shift;
    my ( $table, $column ) = @_;

    #get column definition
    my $TableColumnDefinition = $self->columnDefinition($table, $column);

    #drop auto_increment from column definition
    $TableColumnDefinition =~ s/\sauto_increment//;

    #execute new column definition
    $self->do( "ALTER TABLE $table MODIFY $column $TableColumnDefinition", [] );

    return;
}

#========================================================================================
# function     modifyTableColumnAddAutoIncrement
#----------------------------------------------------------------------------------------
# syntax       $dbi->modifyTableColumnAddAutoIncrement($table, $column );
#----------------------------------------------------------------------------------------
# description  Adds auto_increment of an existing column of an existing table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
#========================================================================================
sub modifyTableColumnAddAutoIncrement {
    my $self = shift;
    my ( $table, $column ) = @_;

    #get column definition
    my $TableColumnDefinition = $self->columnDefinition($table, $column);

    #execute new column definition
    $self->do( "ALTER TABLE $table MODIFY $column $TableColumnDefinition auto_increment", [] );

    return;
}

#========================================================================================
# function     dropIndex
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropIndex($table, $indexname);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $indexname = 'indexname';
#----------------------------------------------------------------------------------------
# description  Drops index $indexname from table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $indexname | index name | string
#========================================================================================
sub dropIndex {
    my $self = shift;
    my ( $table, $indexname ) = @_;

    # get all foreign keys on this indexed column; these must be deleted before and
    # added afterwards to prevent error 103!
    my $aaForeignKeys = $self->_getForeignKeysOnIndexedColumn( $table, $indexname );

    $self->dropForeignKeyIfExists($table, $_->[0]) foreach @$aaForeignKeys;
    $self->do( "ALTER TABLE $table DROP INDEX $indexname", [] );
    $self->addForeignKey($table, $_->[0], [$_->[1]], $_->[2], [$_->[3]]) foreach @$aaForeignKeys;

    return;
}

sub _getForeignKeysOnIndexedColumn {
    my $self = shift;
    my ( $tablename, $indexname ) = @_;

    # Get the indexed column
    my $IndexedColumn = $self->execute( <<'END_SQL', [$indexname] )->[0]->[0];
SELECT column_name FROM information_schema.STATISTICS
WHERE table_schema = DATABASE()
  AND index_name = ?
END_SQL

    # get the foreign keys on this column name
    return $self->execute( <<END_SQL, [ $IndexedColumn, $tablename ] );
SELECT constraint_name, column_name, referenced_table_name, referenced_column_name
FROM information_schema.KEY_COLUMN_USAGE
WHERE column_name = ? and table_name= ?
  AND constraint_name <> 'PRIMARY'
  AND referenced_table_name is not null
  AND CONSTRAINT_SCHEMA = DATABASE()
END_SQL
}

#========================================================================================
# function     dropIndexesIfExist
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->dropIndexesIfExist( $Table, $aIndexes );
#----------------------------------------------------------------------------------------
# description  Drops an index from a table if the given index exists already on the
#               table. Since dropping various indexes in a single SQL statement, an array
#               of indexes is expected as a parameter.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $aIndexes | array of index names to be dropped from $Table | array.string
#========================================================================================
sub dropIndexesIfExist {
    my $self = shift;
    my ($Table, $aIndexes) = @_;

    #check parameter
    TestString( 'Table', $Table );
    TestArray( 'aIndexes', $aIndexes );

    #placeholder
    my @aDroppedIndexes;
    my @aDropStatements;

    #get statement and idexes
    foreach my $Index ( @{ $aIndexes } ) {
        next unless $self->existsIndex( $Table, $Index );
        push @aDropStatements, " DROP INDEX $Index";
        push @aDroppedIndexes, join( '.', $Table, $Index );
    }

    #return if nothing is to do
    return unless @aDropStatements;

    #complete statement
    my $SqlStatement = "ALTER TABLE $Table ";
    $SqlStatement .= join( ',', @aDropStatements );

    #execute statement
    $self->do( $SqlStatement );

    return;
}

#========================================================================================
# function     dropForeignKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropForeignKey($table, $constraint);
#----------------------------------------------------------------------------------------
# example      $table = 'tablename';
#               $constraint = 'fk_tablename_reftablename';
#----------------------------------------------------------------------------------------
# description  Drops constaint (foreign key) $constraint from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $constraint | constraint name | string
#========================================================================================
sub dropForeignKey {
    my $self = shift;
    my ( $table, $constraint ) = @_;
    $self->do( "ALTER TABLE $table DROP FOREIGN KEY $constraint", [] );
    return;
}

#========================================================================================
# function     dropPrimaryKey
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropPrimaryKey($table, $constraint);
#----------------------------------------------------------------------------------------
# example      $dbi->dropPrimaryKey('product', 'pk_product');
#----------------------------------------------------------------------------------------
# description  Drops constaint (Primary key) $constraint from a table $tablename.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $constraint | constraint name (ignored) | string
#========================================================================================
sub dropPrimaryKey {
    my $self = shift;
    my ($table, $constraint) = @_;

    $self->do( "ALTER TABLE $table DROP PRIMARY KEY", [] );
    return;
}

#========================================================================================
# function     dropProcedureIfExists
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropProcedureIfExists($Procedure);
#----------------------------------------------------------------------------------------
# example      $dbi->dropProcedureIfExists('sp_InsertAttribute');
#----------------------------------------------------------------------------------------
# description  Removes a stored procedure from the database if exists
#----------------------------------------------------------------------------------------
# input        $Procedure | procedure name | string
#========================================================================================
sub dropProcedureIfExists {
    my $self = shift;
    my ($ProcedureName) = @_;
    $self->do( "DROP PROCEDURE IF EXISTS $ProcedureName");
    return;
}

#========================================================================================
# function     escapeChar
#----------------------------------------------------------------------------------------
# syntax       $escapeChar = $dbi->escapeChar;
#----------------------------------------------------------------------------------------
# description  Returns the escape character '\' to escape like parameter.
#----------------------------------------------------------------------------------------
# return       $escapeChar | back slash | string
#========================================================================================
sub escapeChar { return '\\'; }

#========================================================================================
# function     getSearchCollation
#----------------------------------------------------------------------------------------
# syntax       $SearchCollation = $dbi->getSearchCollation;
#----------------------------------------------------------------------------------------
# description  Returns additional SQL syntax for specification of collations for searches.
#               This is ususally used for mysql to return
#               'COLLATE utf8_general_ci'
#----------------------------------------------------------------------------------------
# return       $SearchCollation | empty string for standard | string
#========================================================================================
sub getSearchCollation { return 'COLLATE utf8_general_ci'; }

#========================================================================================
# function     dropAllTables
#----------------------------------------------------------------------------------------
# syntax       $dbi->dropAllTables();
#----------------------------------------------------------------------------------------
# description  Removes all user tables from the database (deleting all data, of cause).
#========================================================================================
sub dropAllTables {
    my $self = shift;

    $self->disableForeignKeyChecks( sub {
        # drop all functions
        $self->dropFunction($_) foreach map { $_->[0] } @{ $self->execute(<<END_SQL) };
            SELECT ROUTINE_NAME
            FROM INFORMATION_SCHEMA.ROUTINES
            WHERE ROUTINE_SCHEMA = DATABASE()
            AND ROUTINE_TYPE = 'FUNCTION'
END_SQL

        # drop all procedures
        $self->dropProcedure($_) foreach map { $_->[0] } @{ $self->execute(<<END_SQL) };
            SELECT ROUTINE_NAME
            FROM INFORMATION_SCHEMA.ROUTINES
            WHERE ROUTINE_SCHEMA = DATABASE()
            AND ROUTINE_TYPE = 'PROCEDURE'
END_SQL

        # drop all views
        $self->dropView($_) foreach map { $_->[0] } @{ $self->execute(<<END_SQL) };
            SELECT TABLE_NAME
            FROM INFORMATION_SCHEMA.VIEWS
            WHERE TABLE_SCHEMA = DATABASE()
END_SQL

        # drop all tables
        $self->dropTable($_) foreach map { $_->[0] } @{ $self->execute(<<END_SQL) };
            SELECT TABLE_NAME
            FROM INFORMATION_SCHEMA.TABLES
            WHERE TABLE_SCHEMA = DATABASE()
            AND TABLE_TYPE = 'BASE TABLE'
END_SQL
    });
}

#========================================================================================
# function     getAllTableReferences
#----------------------------------------------------------------------------------------
# syntax       $aaReferences = $dbi->getAllTableReferences();
#----------------------------------------------------------------------------------------
# description  Determines all table references
#----------------------------------------------------------------------------------------
# return       $aaReferences | list of table references
#                       each list element is a pair of tables [ table => master_table ].
#                       If master_table is undef, then the table does not depend on other
#                       tables | ref.array.array.string
#========================================================================================
sub getAllTableReferences {
    my $self = shift;

    return $self->execute( <<END_SQL );
SELECT DISTINCT k.TABLE_NAME, k.REFERENCED_TABLE_NAME
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS c
INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE k ON( c.TABLE_NAME = k.TABLE_NAME
                                                AND c.CONSTRAINT_NAME = k.CONSTRAINT_NAME
                                                AND c.CONSTRAINT_SCHEMA = k.CONSTRAINT_SCHEMA )
WHERE c.CONSTRAINT_TYPE = 'FOREIGN KEY'
  AND c.CONSTRAINT_SCHEMA = DATABASE()
ORDER BY k.TABLE_NAME, k.REFERENCED_TABLE_NAME
END_SQL
}

#========================================================================================
# function     atIsolationLevel
#----------------------------------------------------------------------------------------
# syntax       $dbi->atIsolationLevel($IsolationLevel, $Sub);
#----------------------------------------------------------------------------------------
# description  executes the function sub at specified transaction isolation level.
#----------------------------------------------------------------------------------------
# input        $IsolationLevel | values (
#                       READ UNCOMMITTED|
#                       READ COMMITTED|
#                       REPEATABLE READ|
#                       SERIALIZABLE
#               ) | string
# input        $Sub | function to execute select statement | ref.code
#========================================================================================
sub atIsolationLevel {
    my $self = shift;
    my ( $IsolationLevel, $Sub ) = @_;

    my $OldIsolationLevel = $self->execute('select @@tx_isolation')->[0]->[0]; ##no critic(RequireInterpolationOfMetachars)
    $OldIsolationLevel =~ s/-/ /g;    # @@tx_isolation returns "REPEATABLE-READ"

    $self->do("SET TRANSACTION ISOLATION LEVEL $IsolationLevel");
    $Sub->();
    $self->do("SET TRANSACTION ISOLATION LEVEL $OldIsolationLevel");
}

#========================================================================================
# function     disableForeignKeyChecks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->disableForeignKeyChecks($Code);
#----------------------------------------------------------------------------------------
# description  Disables foreign key checks during execution of $Code.
#----------------------------------------------------------------------------------------
# input        $Code | code reference | code
#========================================================================================
sub disableForeignKeyChecks {
    my $self = shift;
    my ($Code) = @_;

    $self->do('SET FOREIGN_KEY_CHECKS=0');
    eval {
        $Code->();
    };
    my $Error = ExistsError() ? GetError() : undef;
    $self->do('SET FOREIGN_KEY_CHECKS=1');
    Error($Error) if $Error;

    return;
}

#========================================================================================
# function     addTableColumnFirst
#----------------------------------------------------------------------------------------
# syntax       $dbi->addTableColumnFirst($table, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $dbi->addTableColumnFirst('productprice', 'currencyid CHAR(3) NULL');
#----------------------------------------------------------------------------------------
# description  Adds a new column to an existing table $table, before all existing columns.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $columndefinition | sql definiton of the column | string
#========================================================================================
sub addTableColumnFirst {
    my $self = shift;
    my ($table, $columndefinition) = @_;
    $self->do("ALTER TABLE $table ADD $columndefinition FIRST", []);
    return;
}

#========================================================================================
# function     addTableColumnAfterColumn
#----------------------------------------------------------------------------------------
# syntax       $dbi->addTableColumnAfterColumn($table, $column, $columndefinition);
#----------------------------------------------------------------------------------------
# example      $dbi->addTableColumnAfterColumn('product', 'weighunitid', 'manufacturerid INT NULL');
#----------------------------------------------------------------------------------------
# description  Adds a new column to an existing table $table, after a specified existing column.
#----------------------------------------------------------------------------------------
# input        $table | table name | string
# input        $column | column name | string
# input        $columndefinition | sql definiton of the column | string
#========================================================================================
sub addTableColumnAfterColumn {
    my $self = shift;
    my ($table, $column, $columndefinition) = @_;
    $self->do("ALTER TABLE $table ADD $columndefinition AFTER $column", []);
    return;
}

#========================================================================================
# function     truncateTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $dbi->truncateTable( $Table );
#----------------------------------------------------------------------------------------
# description  Delete all table rows without where clause.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
#========================================================================================
sub truncateTable {
    my $self = shift;
    my ($Table) = @_;

    TestString('Table', $Table);
    $self->do("TRUNCATE TABLE $Table");
    return;
}

1;
