#========================================================================================
# package      DE_EPAGES::LeguideCom::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::LeguideCom::API::Cartridge;
use base DE_EPAGES::ProductPortal::API::PortalInstaller;

use strict;
use DE_EPAGES::Core::API::File qw (DeleteDirectory);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (GetInfoCartridgeByPackage ExistsCartridgeByPackage);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadObjectByPath);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::LeguideCom::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::LeguideCom.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/LeguideCom',
        'Version'            => '7.44.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    my $ExistsOldCartridge = ExistsCartridgeByPackage('COM_LEGUIDE::LeguideCom');
    if ($ExistsOldCartridge) {
        # check if old cartridge has version 6.9.0
        my $OldLeguide = $self->loadInstaller('COM_LEGUIDE::LeguideCom');
        my $Version = $OldLeguide->{'Version'};
        if ($Version ne '6.9.0') {
            die("DE_EPAGES::LeguideCom can't be installed; COM_LEGUIDE::LeguideCom installed with version $Version, required: 6.9.0");
        }
    }
    $self->startInstall;

    if ($ExistsOldCartridge) {
        # reset ftp credentials for all portal configs - not needed any longer
        my $Portal = LoadObjectByPath('/Portals/LeguideCom');
        my $dbi = GetCurrentDBHandle();
        my $sql = 'update portalconfig set ftplogin=?, ftppassword=? where portalid=?';
        $dbi->do($sql, [undef, undef, $Portal->id]);

        # reset ftp connection data for all portal sites
        $sql = 'update portalsite set ftpserver=?, ftpport=? where portalid=?';
        $dbi->do($sql, [undef, undef, $Portal->id]);

        # update feature to avoid removing of feature during uninstallation
        if (ExistsObjectByPath('/Features/LeguideCom')) {
            my $Feature = LoadObjectByPath('/Features/LeguideCom');
            $Feature->set({'CartridgeID' => GetInfoCartridgeByPackage('DE_EPAGES::ProductPortal')->{'CartridgeID'}});
        }

        # we can uninstall the old cartridge because we modified it so that all the data that are still necessary remain in the db
        my $OldLeguide = $self->loadInstaller('COM_LEGUIDE::LeguideCom');
        {
            local $ENV{'EPAGES_BACKUP_DIR'} = '';
            $OldLeguide->uninstall;
        }
# we can not remove the cartridge directory as we need it for uninstallation of further stores
#        DeleteDirectory("$ENV{'EPAGES_CARTRIDGES'}/COM_LEGUIDE", {'recursive' => 1});

        # remove webroot directories
        foreach my $langCode (qw(de en es fr it nl)) {
            DeleteDirectory("$ENV{'EPAGES_WEBROOT'}/Doc/Help/$langCode/leguide", {'recursive' => 1});
        }
    }

    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

1;
