#========================================================================================
# package      DE_EPAGES::Order::t::unit::ProductValidator
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::ProductValidator
#========================================================================================
package DE_EPAGES::Order::t::unit::ProductValidator;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE STOCK_NOT_VISIBLE );
use DE_EPAGES::Order::API::ProductValidator qw ( ValidateStockLevelSettings ValidateAvailibilityVisibilitySettings );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::More;
use Test::Exception;

use strict;

sub test {
    my $self = shift;

    $self->test_stockLevelRestriction_success();
    $self->test_stockLevelRestriction_failed();
    $self->test_stockAvailabilityRestriction_success();
    $self->test_stockAvailabilityRestriction_failed();

    done_testing();
}
#------------------------------------------
sub test_stockLevelRestriction_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LowStockSetting = STOCK_IS_ORDERABLE;
    my $Shop = $self->_createShop($LowStockSetting);
    my $Product = $self->_createProduct($Shop);
    my $RequestQuantity = 3;

    lives_ok( sub { ValidateStockLevelSettings($Product, $RequestQuantity); },
        "$SubTestName - Test if it is not throwing an error on low stocklevel" )
}

#------------------------------------------
sub test_stockLevelRestriction_failed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LowStockSetting = STOCK_NOT_VISIBLE;
    my $Shop = $self->_createShop($LowStockSetting);
    my $Product = $self->_createProduct($Shop);
    my $RequestQuantity = 12; ## no critic (ProhibitMagicNumbers)

    throws_ok(sub {ValidateStockLevelSettings($Product, $RequestQuantity);},
    qr/Low StockLevel /, ## no critic (ProhibitComplexRegexes)
         "$SubTestName - Test if throws error in Low StockLevel");

}

#------------------------------------------
sub test_stockAvailabilityRestriction_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_createShop();
    my $Product = $self->_createProduct($Shop);
    $Product->set({ 'IsVisible'   => 1,
                    'IsAvailable' => 1,
    });

    lives_ok( sub { ValidateAvailibilityVisibilitySettings($Product); },
        "$SubTestName - Test if it is not throwing an error on available product" )
}

#------------------------------------------
sub test_stockAvailabilityRestriction_failed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_createShop();
    my $Product = $self->_createProduct($Shop);

    $Product->set({ 'IsVisible'   => 0,
                    'IsAvailable' => 1,
    });

    $self->_doVisibilityTest($Product, $SubTestName, "Test if throws error in Not Visible");

    $Product->set({ 'IsVisible'   => 1,
                    'IsAvailable' => 0,
    });

    $self->_doVisibilityTest($Product, $SubTestName, "Test if throws error in Not Available");

    $Product->set({ 'IsVisible'   => 0,
                    'IsAvailable' => 0,
    });

    $self->_doVisibilityTest($Product, $SubTestName, "Test if throws error in Not Available and Not Visible");
}

#------------------------------------------
### helper methods ###
#------------------------------------------
sub _doVisibilityTest {
    my $self = shift;
    my ($Product, $SubTestName, $Text) = @_;

    throws_ok(sub {ValidateAvailibilityVisibilitySettings($Product);},
    qr/Not AvailableOrVisible /, ## no critic (ProhibitComplexRegexes)
         "$SubTestName - $Text");
}

sub _createProduct {
    my $self = shift;
    my ($Shop) = @_;

    my $hAttributes = {
        'StockLevel' => 10,
        'Site' => $Shop,
        'IsVisible' => 0,
        'IsAvailable' => 0,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Product::API::Object::Product', $hAttributes );
    my $Product = $Mockify->getMockObject();
    return $Product;
}
#------------------------------------------
sub _createShop {
    my $self = shift;
    my ($LowStockSetting) = @_;

    my $hAttributes = { 'ProductLowOnStockAction' => $LowStockSetting, };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();