#==============================================================================
# package      DE_EPAGES::Presentation::API::TLE::LocaleHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the TLE function "ChangeLocaleURL".
#========================================================================================
package DE_EPAGES::Presentation::API::TLE::LocaleHandler;

use strict;

use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw(LoadObject ExistsObject);
use DE_EPAGES::Presentation::API::LocaleHandler qw ( AddLanguageToURL );
use DE_EPAGES::Presentation::API::Session qw(GetSessionAttribute);
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );
use URI::Escape qw( uri_escape_utf8 );

sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'LocaleHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'LocaleHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'ChangeLocaleURL');
    $Processor->registerHandler('FunctionHandler', $self, 'AddLanguageToUnitySfURL');
    return;
}

#========================================================================================
# function     ChangeLocaleURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ChangeLocaleURL(#LocaleID)
#----------------------------------------------------------------------------------------
# description  Returns url with changed locale.
#----------------------------------------------------------------------------------------
# input        #LocaleID | locale id | string
# return       #URL | url with changed locale| string
#========================================================================================
sub ChangeLocaleURL { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($LocaleID) = @$aParams;

    my $IntSessionID    = $Processor->tle('INPUT.IntSessionID');
    my $User            = $IntSessionID ? GetSessionAttribute( $IntSessionID, 'User' ) : undef;
    my $hPager          = $Processor->tle('Pager');
    my $ViewObjectID    = $Processor->tle('INPUT.ViewObjectID');
    my $LastObjectID    = $Processor->tle('INPUT.LastObjectID');
    my $LastViewAction  = $Processor->tle('INPUT.LastViewAction');
    my $ViewAction      = $Processor->tle('INPUT.ViewAction');
    my $HasViewObjectID = defined $ViewObjectID && ExistsObject($ViewObjectID) ? 1 : 0;

    my $Url = $IntSessionID ? "?Locale=$LocaleID&" : "../$LocaleID/?";
    if($hPager){
        my $UrlPage = $hPager->{'URLPage'};
        $UrlPage =~ s/\?//igx;
        $Url .=  $UrlPage;
        if($HasViewObjectID) {
            my $ObjectPath = uri_escape_utf8( LoadObject($ViewObjectID)->get('Path') );
            $Url .= "&ViewObjectPath=$ObjectPath";
        }
    } else {
        my $ObjectPath;
        if(defined $LastObjectID && ExistsObject($LastObjectID)) {
            $ObjectPath = uri_escape_utf8( LoadObject($LastObjectID)->get('Path') );
        } elsif ($HasViewObjectID) {
            $ObjectPath = uri_escape_utf8( LoadObject($ViewObjectID)->get('Path') );
        }
        $Url .= 'ViewObjectPath='.$ObjectPath if defined $ObjectPath;

        if($LastViewAction and $LastViewAction ne 'View'){
            $LastViewAction = uri_escape_utf8( $LastViewAction );
            $Url .= "&ViewAction=$LastViewAction&";
        } elsif($ViewAction ne 'View') {
            $ViewAction = uri_escape_utf8( $ViewAction );
            $Url .= "&ViewAction=$ViewAction";
        }
    }

    if($User){
        $Url .= '&ChangeAction=SaveLocale&ChangeObjectID='.$User->id;
    }

    # triggering hook for EPG-19186: SF: Blog: broken link when switching language on ViewTag page of a blog

    my $QueryString = $Processor->tle('INPUT.QUERY_STRING');
    TriggerHook('API_UrlLocale', {'Url' => \$Url, 'QueryString' => \$QueryString});

    return $Url;
}

#========================================================================================
# function     AddLanguageToUnitySfURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       #AddLanguageToUnitySfURL(#URL, #AltLanguageID, #LanguageID)
#----------------------------------------------------------------------------------------
# description  Converts a given AltLanguageID to the fitting language code and adds this
#               to the given URL if it is different to the given LanguageID. If the
#               optional user is registered, a switch to that users language will be
#               performed instead.
#----------------------------------------------------------------------------------------
# input        #URL | Unity storefront URL | string
# input        #AltLanguageID | language id | string
# input        #LanguageID | language id | string
# input        #User | (optional) User object | object
# return       #URL | url with added locale | string
#========================================================================================
sub AddLanguageToUnitySfURL {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($UnityStorefrontURL, $AltLanguageID, $LanguageID, $User) = @$aParams;

    # If the user is registered, use that language instead.
    if (defined($User)) {
        TestObject('User', $User, 'DE_EPAGES::Permission::API::Object::User');
        my $UserLanguageID = $User->get('LanguageID');
        return AddLanguageToURL($UnityStorefrontURL, $UserLanguageID, $LanguageID);
    }
    return AddLanguageToURL($UnityStorefrontURL, $AltLanguageID, $LanguageID);
}

1;
