﻿#!/usr/bin/perl

#============================================================================
# Author: Oliver Zscheyge
# Description:
#     Find all duplicate language tags.
#============================================================================

use utf8;
use strict;
use warnings;

use Data::Dumper;
use XML::Twig;

# UTF-8 output
binmode(STDOUT, ":encoding(UTF-8)");

# Print usage
if (@ARGV != 1 or $ARGV[0] eq "-h" or $ARGV[0] eq "--help") {
    print "Prints duplicate language tag definitions (requires epages Perl).\n\n";
    print "Usage:\n";
    print "  checkDuplicateTagDefs.pl <file or directory to check>\n\n";
    print "Example usage: \$PERL checkDuplicateTagDefs.pl \$EPAGES_CARTRIDGES/DE_EPAGES\n";
    exit;
}

# Find all .de.xml files
my $SearchPath = $ARGV[0];
my @FilesToCheck = findLanguageFiles($SearchPath);
@FilesToCheck = sort @FilesToCheck;

# Find duplicate definitions within the translation definitions
my @Duplicates = ();
# Find all translations in those XML files
my @Definitions;
foreach my $FileToCheck (@FilesToCheck) {
    @Definitions = ();
    my $twig = XML::Twig->new(twig_handlers =>
                                {
                                    "Language" => handle_language($FileToCheck)
                                });
    $twig->parsefile($FileToCheck);

    foreach my $Definition (@Definitions) {
        my @Occurrences = ();
        foreach my $Other (@Definitions) {
            # keywords are the same
            if ($Definition->[1] eq $Other->[1]) {
                push @Occurrences, $Definition
            }
        }
        if (scalar @Occurrences > 1) {
            push @Duplicates, @Occurrences
        }
    }
}

# Only choose unique occurrences, sort by file and tag name
@Duplicates = sort {($a->[0] cmp $b->[0]) + ($a->[1] cmp $b->[1])} unique(@Duplicates);

# Print the duplicates by file name
my $LastFile = "";
foreach my $Duplicate (@Duplicates) {
    if ($LastFile ne "" and $LastFile ne $Duplicate->[0]) {
        print "\n";
    }
    print $Duplicate->[0] . ", " . $Duplicate->[1] . ", " . $Duplicate->[2] . "\n";
    $LastFile = $Duplicate->[0];
}

if (scalar @Duplicates > 0) {
    exit 1
} else {
    exit 0
}

#============================================================================
# §function     findLanguageFiles
#----------------------------------------------------------------------------
# §syntax       @FilesToCheck = findLanguageFiles($FileOrDir);
#----------------------------------------------------------------------------
# §example      @FilesToCheck = findLanguageFiles($FileOrDir);
#----------------------------------------------------------------------------
# §description  Recursively finds all .de.xml files in a given path.
#----------------------------------------------------------------------------
# §input        $FileOrDir | Search path, can point to a file or dir | string
# §return       @LanguageFiles | Array of .de.xml language file names | array
#============================================================================
sub findLanguageFiles {
    my $FileOrDir = shift;

    my @LanguageFiles = ();
    if (-d $FileOrDir) {
        # Directory
        opendir(DirHandle, "$FileOrDir") || die "Can't open directory $FileOrDir: $!\n";
        my @DirContents = readdir(DirHandle);
        closedir(DirHandle);

        foreach my $DirContent (@DirContents) {
            unless ($DirContent =~ /^\.\.?$/) {
                # Ignore . and .. directories to avoid infinite recursion
                push @LanguageFiles, findLanguageFiles($FileOrDir . "/" . $DirContent);
            }
        }
    } else {
        # Regular file ending with .<language-code>.xml
        if ($FileOrDir =~ /\.(ca|cs|da|de|en-US|en|es-MX|es|fi|fr|it|nl|no|pt-BR|pt|ru|sv)\.xml$/) {
            push @LanguageFiles, $FileOrDir;
        }
    }

    @LanguageFiles;
}

#============================================================================
# §function     handle_language
#----------------------------------------------------------------------------
# §syntax       handle_language($FileToCheck);
#----------------------------------------------------------------------------
# §example      handle_language($FileToCheck);
#----------------------------------------------------------------------------
# §description  Fills the global @Definitions array with triples of
#               file, tag, translation
#----------------------------------------------------------------------------
# §input        $FileToCheck | File context | string
# §return       | Anonymous twig element handler | sub
#============================================================================
sub handle_language {
    my ($FileToCheck) = shift;

    return sub {
        my ($twig, $Language) = @_;

        my @Translations = $Language->children("Translation");

        foreach my $Translation (@Translations) {
            my $Keyword = $Translation->att("Keyword");
            my $TranslationText = $Translation->text;
            push @Definitions, [$FileToCheck, $Keyword, $TranslationText];
        }
    };
}

#============================================================================
# §function     unique
#----------------------------------------------------------------------------
# §syntax       @Uniques = unique(@Duplicates);
#----------------------------------------------------------------------------
# §example      @Uniques = unique(@Duplicates);
#----------------------------------------------------------------------------
# §description  Returns an array of unique elements from a given array.
#----------------------------------------------------------------------------
# §input        @Duplicates | Array that can contain duplicate entries | array
# §return       @Unique | Array containing only unique elements | array
#============================================================================
sub unique {
    my %seen;
    grep {! $seen{ Dumper $_ }++}  @_
}
