﻿#========================================================================================
# §package      DE_EPAGES::Shop::API::TLE::UrlHandler
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements system-wide TLE variables for links that can't be saved as
#               static values in the DB.
#               <ul>
#                   <li>HelpCenterUrl</li>
#               </ul>
#========================================================================================
package DE_EPAGES::Shop::API::TLE::UrlHandler;

use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Log qw (GetLog LogError);
use DE_EPAGES::Dictionary::API::Template;
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);

use strict;
use utf8;

#========================================================================================
# §function     HelpCenterUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       HelpCenterUrl( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# §description  TLE function to provide link to help center.
#----------------------------------------------------------------------------------------
# §input        $Proccessor | tle processor | object
# §input        $aParams | tle parameter | ref array
# §return       $HelpCenterUrl | url to help center | url
#========================================================================================
sub HelpCenterUrl {
    my $self = shift;
    my ( $Processor, $aParams ) = @_;

    my $HelpCenterUrl = eval {
        my $System        = LoadRootObject();
        my $User          = $aParams->[0];
        return $self->getHelpCenterUrl( $System, $User );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
        $HelpCenterUrl = '';
    }

    return $HelpCenterUrl;
}

#========================================================================================
# §function     getHelpCenterUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       getHelpCenterUrl( $System, $User );
#----------------------------------------------------------------------------------------
# §description  Method to provide url to help center.
#----------------------------------------------------------------------------------------
# §input        $System | system | object
# §input        $User | user | object
# §return       $HelpCenterUrl | url to help center | url
#========================================================================================
sub getHelpCenterUrl {
    my $self = shift;
    my ( $System, $User ) = @_;

    my $LanguageID     = $User->get('LanguageID');
    my $HelpLanguageID = $self->_getHelpLanguageID($LanguageID);
    my $HelpCenterUrl  = $System->get(
        $User->getSite()->get('IsUnityRegistered')
            ? 'HelpCenterUrlUnity'
            : 'HelpCenterUrl'
    );

    $HelpCenterUrl =~ s/<LanguageCode2>/$HelpLanguageID/g;

    return $HelpCenterUrl;
}

sub _getHelpLanguageID {
    my $self = shift;
    my ($LanguageID) = @_;

    my $hLanguageInfo       = $self->_getInfoLanguage($LanguageID);
    my $LanguageCode2       = $hLanguageInfo->{'Code2'};
    my $DictionaryXml       = $ENV{'EPAGES_CARTRIDGES'}."/DE_EPAGES/Dictionary/Templates/Dictionary.$LanguageCode2.xml";
    my $Template            = $self->_getTemplate();
    my $hDictionary         = $Template->dictionary( $DictionaryXml, $LanguageCode2 );
    my $hTranslationStrings = $hDictionary->{$LanguageCode2};

    return $hTranslationStrings->{'HelpLanguageID'};
}

sub _getInfoLanguage {
    my $self = shift;
    my ($LanguageID) = @_;

    return GetInfoLanguage($LanguageID);
}

sub _getTemplate {
    my $self = shift;

    return DE_EPAGES::Dictionary::API::Template->new();
}

#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug('UrlHandler.RegisterHandlerProc');

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::Object::API::TLE::UrlHandler;
# §example      my $UrlHandler = new DE_EPAGES::Object::API::TLE::UrlHandler;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new {
    return bless {}, shift;
}

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    GetLog->debug('UrlHandler.register');

    $Processor->registerHandler( 'FunctionHandler', $self, 'HelpCenterUrl' );

    return;
}

1;
